import { createContext } from 'react';
/**
 * React context which holds ProstoreSsrManager instance
 */
export var ProstoreSsrContext = createContext(null);
/**
 * Globally manages the store states for them to be used in the SSR-enabled environments
 */
var ProstoreSsrManager = /** @class */ (function () {
    function ProstoreSsrManager(_a) {
        var isServer = _a.isServer, initialStates = _a.initialStates;
        /** Whether the application is running on the server or in the browser */
        this.isServer = false;
        /** Store states identified by store's ssrId */
        this.states = {};
        /** Array of registered promises to be awaited during the SSR */
        this.promises = new Set();
        /** Do nothing if this is true. setState will do nothing and getState will return null */
        this.disabled = false;
        this.isServer = isServer || false;
        this.states = initialStates || this.states;
    }
    /**
     * Registers the promise from the store for it to be awaited during SSR
     *
     * @param promise - promise to be registered
     */
    ProstoreSsrManager.prototype.registerPromise = function (promise) {
        var _this = this;
        this.promises.add(promise);
        promise.then(function () {
            _this.promises.delete(promise);
        });
    };
    /**
     * Checks if it has any registered promises
     */
    ProstoreSsrManager.prototype.hasPromises = function () {
        return this.promises.size > 0;
    };
    /**
     * Awaits all current promises
     */
    ProstoreSsrManager.prototype.waitForCurrentPromises = function () {
        return Promise.all(this.promises);
    };
    /**
     * Returns the saved state for the store with the provided ssrId
     * @param key - ssrId of the store
     * @returns saved state for the provided store
     */
    ProstoreSsrManager.prototype.getState = function (key) {
        if (this.disabled)
            return null;
        return this.states[key];
    };
    /**
     * Saves the state of the store with the provided ssrId
     * @param key - ssrId of the store
     * @param value - state to be saved
     */
    ProstoreSsrManager.prototype.setState = function (key, value) {
        if (this.disabled)
            return;
        this.states[key] = value;
    };
    /**
     * Returns all store states
     */
    ProstoreSsrManager.prototype.getStates = function () {
        return this.states;
    };
    /**
     * Changes the disabled flag
     * @param value - new value of the flag
     */
    ProstoreSsrManager.prototype.setDisabled = function (value) {
        this.disabled = value;
    };
    return ProstoreSsrManager;
}());
export { ProstoreSsrManager };
//# sourceMappingURL=ProstoreSsrContext.js.map