import { __assign } from "tslib";
import { useCallback, useState } from 'react';
import { useLatestCallbackRef, useMountedRef, usePropsRef, useTimeoutRef } from '@proscom/ui-react';
var changeLoading = function (change) { return function (state) { return (__assign(__assign({}, state), { loading: state.loading + change })); }; };
var changeFinished = function (finished) { return function (state) { return (__assign(__assign({}, state), { finished: finished })); }; };
/**
 * Allows to observe the status of async operation (loading, finished)
 * @param callback - async callback which performs the operation
 * @param options - hook options:
 *  - singleton - prevents calling operation until previous call is finished
 *  - finishedTimeout - if set, after operation is completed, sets value of
 *      finished to true for the given number of milliseconds. To set finished
 *      permanently, set finishedTimeout to Infinity
 * @example
 *  const [run, loading] = useAsyncOperation(() => fetch('url').catch(e => console.error(e)));
 *  return <div>{loading ? 'Loading...' : <button onClick={run} />}
 */
export function useAsyncOperation(callback, options) {
    if (options === void 0) { options = {}; }
    var _a = useState({
        loading: 0,
        finished: false
    }), state = _a[0], setState = _a[1];
    var timeoutRef = useTimeoutRef();
    var mountedRef = useMountedRef();
    var callbackRef = useLatestCallbackRef(callback);
    var latestRef = usePropsRef({
        loading: state.loading,
        options: options
    });
    var run = useCallback(function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var options = latestRef.options, loading = latestRef.loading;
        if (options.singleton && loading > 0) {
            return Promise.resolve();
        }
        setState(changeLoading(1));
        return callbackRef.apply(void 0, args).then(function (result) {
            if (mountedRef.current) {
                setState(changeLoading(-1));
                if (options.finishedTimeout !== undefined &&
                    options.finishedTimeout > 0) {
                    setState(changeFinished(true));
                    if (options.finishedTimeout !== Infinity) {
                        timeoutRef.set(function () {
                            setState(changeFinished(false));
                        }, options.finishedTimeout);
                    }
                }
            }
            return result;
        }, function (e) {
            if (mountedRef.current) {
                setState(changeLoading(-1));
            }
            throw e;
        });
    }, [latestRef, timeoutRef]);
    var setFinished = useCallback(function (finished) {
        setState(changeFinished(finished));
    }, []);
    return {
        run: run,
        loading: state.loading > 0,
        finished: state.finished,
        setFinished: setFinished
    };
}
//# sourceMappingURL=useAsyncOperation.js.map