import { useContext, useEffect, useState } from "react";
import { checkRequestState } from "@proscom/prostore";
import { useLatestCallbackRef } from "@proscom/ui-react";
import { useConnectStore } from "./useConnectStore";
import { useContextStore } from "./ProstoreContext";
import { ProstoreSsrContext } from "./ProstoreSsrContext";
/**
 * Attaches component to the provided store and requests data based on the variables and options
 *
 * @param storeOrName - RequestStore to subscribe to, or its name for it to be injected from the context
 * @param variables - Request variables. If they change, request will be fired again
 * @param options - Additional request options. They are not observed
 * @param hookOptions - Additional hook options:
 *  - checkRequest - Optional function which may be used to override default behavior of determining
 *                   when to fire the request
 */
export function useRequestStore(storeOrName, variables, options, hookOptions) {
    if (variables === void 0) { variables = undefined; }
    if (options === void 0) { options = undefined; }
    if (hookOptions === void 0) { hookOptions = {}; }
    var checkRequest = hookOptions.checkRequest || checkRequestState;
    // Inject store from context
    var store = useContextStore(storeOrName);
    // Local copy of store state
    var _a = useState(store.state), state = _a[0], setState = _a[1];
    // Subscribe to store changes
    useConnectStore(store, setState);
    // Check if we need to request data
    var queryCheck = checkRequest(state, variables);
    // Memoized callback to request data
    var loadData = useLatestCallbackRef(function (newOptions) {
        if (newOptions === void 0) { newOptions = undefined; }
        return store.loadData(variables, newOptions || options);
    });
    // Request the data if it is should be requested
    useEffect(function () {
        if (queryCheck.request) {
            // Error should not happen here, because
            // it is handled inside RequestStore,
            // but we attach the catch listener just in case
            loadData().catch(function (e) { return console.error(e); });
        }
    });
    // Attach ssrContext
    var ssrContext = useContext(ProstoreSsrContext);
    // Perform request if server rendering
    if (store.ssrId && ssrContext && ssrContext.isServer && queryCheck.request) {
        loadData().catch(function (e) { return console.error(e); });
    }
    // Return stuff which may be useful in the component
    return { check: queryCheck, state: state, load: loadData, store: store };
}
//# sourceMappingURL=useRequestStore.js.map